/***
  GpsConnectorLib.cs
  GPS𓮂Cu
  
  for C#2.0(VisualStudio2005)
  Date: 2007/1/4
  Author: Sho Hashimoto
  WebSite: http://shokai.org
 ***/

using System;
using System.Collections.Generic;
using System.Text;
using System.IO.Ports;

namespace Org.Shokai.Util
{

    /// <summary>
    /// k/씼\񋓑
    /// </summary>
    public enum HemiSpheriumNS
    {
        /// <summary>
        /// k
        /// </summary>
        North,
        /// <summary>
        /// 씼
        /// </summary>
        South
    }

    /// <summary>
    /// /\񋓑
    /// </summary>
    public enum HemiSpheriumEW
    {
        /// <summary>
        /// 
        /// </summary>
        East,
        /// <summary>
        /// 
        /// </summary>
        West
    }

    /// <summary>
    /// GPS Information Data
    /// </summary>
    class GpsData{
        /// <summary>
        /// ox
        /// </summary>
        public double lat = Int32.MinValue; // ܓx
        /// <summary>
        /// ܓx
        /// </summary>
        public double lon = Int32.MinValue; // ox
        /// <summary>
        /// k/씼\񋓑
        /// </summary>
        public HemiSpheriumNS NS;
        /// <summary>
        /// /\񋓑
        /// </summary>
        public HemiSpheriumEW EW;
        public GpsData()
        {
            NS = HemiSpheriumNS.North; // k
            EW = HemiSpheriumEW.East; // 
        }
    }

    class GpsConnector
    {
        public GpsData gpsData = new GpsData();

        public GpsConnector()
        {
            serialPort = new SerialPort();
            PortName = "COM1";
            BaudRate = 4800;
            serialPort.DataReceived += serialPort_DataReceived;
            hasError = true;
        }

#region Property and Accessor

        public Boolean hasError;
        private SerialPort serialPort;

        public String PortName
        {
            set
            {
                this.serialPort.PortName = value;
            }
            get
            {
                return this.serialPort.PortName;
            }
        }

        public int BaudRate
        {
            set
            {
                this.serialPort.BaudRate = value;
            }
            get
            {
                return this.serialPort.BaudRate;
            }
        }

        public Boolean IsOpen
        {
            get
            {
                return this.serialPort.IsOpen;
            }
        }

#endregion

#region Method
        public Boolean connect()
        {
            if (IsOpen) return false; // ɊJĂ
            try
            {
                serialPort.Open();
            }
            catch
            {
            }
            return serialPort.IsOpen;
        }

        public Boolean close()
        {
            if (!IsOpen) return false; // ɕĂ
            try
            {
                serialPort.Close();
            }
            catch
            {
            }
            return serialPort.IsOpen;
        }

        public String toString()
        {
            String gpsStat = "";
            if (hasError) gpsStat += "G[L\r\n";
            else gpsStat += "G[\r\n";
            gpsStat += "ܓx:" + gpsData.lon + " ox:" + gpsData.lat;

            if (gpsData.EW == HemiSpheriumEW.East)
            {
                gpsStat += ",";
            }
            else if (gpsData.EW == HemiSpheriumEW.West)
            {
                gpsStat += ",";
            }
            if (gpsData.NS == HemiSpheriumNS.North)
            {
                gpsStat += ",k";
            }
            else if (gpsData.NS == HemiSpheriumNS.South)
            {
                gpsStat += ",씼";
            }
            gpsStat += "\r\n";

            gpsStat += "Port:"+PortName+",BaudRate:"+BaudRate;

            return gpsStat;
        }

#endregion

#region Event
        private void serialPort_DataReceived(object sender, System.IO.Ports.SerialDataReceivedEventArgs e)
        {
            String recvData = serialPort.ReadLine();

            // parse Data from GPS
            String[] data = recvData.Split(',');
            switch (data[0])
            {
                #region case "$CPGGA":
                case "$GPGGA":
                    hasError = false; // reset error flag

                    if (data[2] != "") // ܓx
                    {
                        gpsData.lon = Double.Parse(data[2]) / 100;
                    }
                    else
                    {
                        gpsData.lon = Int32.MinValue;
                        hasError = true;
                    }

                    if (data[3] == "N") // k
                    {
                        gpsData.NS = HemiSpheriumNS.North;
                    }
                    else if (data[3] == "S") // 씼
                    {
                        gpsData.lon *= -1; // 
                        gpsData.NS = HemiSpheriumNS.South;
                    }
                    else hasError = true;

                    if (data[4] != "") // ox
                    {
                        gpsData.lat = Double.Parse(data[4]) / 100;
                    }
                    else
                    {
                        gpsData.lat = Int32.MinValue;
                        hasError = true;
                    }

                    if (data[5] == "E") // 
                    {
                        gpsData.EW = HemiSpheriumEW.East;
                    }
                    else if (data[5] == "W") // 
                    {
                        gpsData.lat *= -1; // o
                        gpsData.EW = HemiSpheriumEW.West;
                    }
                    else hasError = true;

                    break;
                #endregion
                case "$GPRMC":
                    break;
                case "$GPGSA":
                    break;
            }
            Console.WriteLine(recvData);
        }
#endregion

    }
}
